'///***************************************************************************
'// File:		MemoryPool.h
'//
'// Purpose:		Class definition for the memory pool class used by the
'//				memory manager.  Each pool is a block of memory set
'//				aside for a specific thread for use in creating temporary
'//				XLOPER/XLOPER12's in the framework
'// 
'// Platform:    Microsoft Windows
'//
'///***************************************************************************

'#include <windows.h>

'//
'// Total amount of memory to allocate for all temporary XLOPERs
'//

CONST MEMORYSIZE   =  10240

'class MemoryPool
'{
'public:
'	MemoryPool(void);
'	~MemoryPool(void);
'	void ClearPool(void);
'	LPSTR GetTempMemory(size_t cBytes);
'	void FreeAllTempMemory();

'	DWORD m_dwOwner;			// ID of ownning thread
'	char* m_rgchMemBlock;		// Memory for temporary XLOPERs
'	size_t m_ichOffsetMemBlock;	// Offset of next memory block to allocate
'};
#pragma once
#include once "windows.bi"

TYpe MemoryPool    'extends  object
  
 public:
     declare constructor()
     declare destructor()
     declare sub ClearPool()
     declare function GetTempMemory(ctbytes as size_t) as lpstr
     Declare sub FreeAllTempMemory()
     dim m_dwOwner as DWORD    '// ID of ownning thread
    dim  m_rgchMemBlock as zstring ptr		'// Memory for temporary XLOPERs
    dim  m_ichOffsetMemBlock as size_t '	// Offset of next memory block to allocate
end type

'//***************************************************************************
'// File:        MemoryPool.cpp
'//
'// Purpose:     A memory pool is an array of characters that is pre-allocated,
'//              and used as temporary memory by the caller. The allocation
'//              algorithm is very simple. When a thread asks for some memory,
'//              the index into the array moves forward by that many bytes, and
'//              a pointer is returned to the previous index before the pointer
'//              was advanced. When a call comes to free all of the memory, the
'//              pointer is set back to the beginning of the array.
'//
'//              Each pool has MEMORYSIZE bytes of storage space available to it
'// 
'// Platform:    Microsoft Windows
'//
'///***************************************************************************

'#include "MemoryPool.h"
 '***********************************************************************************
'//
'// Constructor creates the memory to be used by the pool
'// and starts the index at the beginning.
'//
'***********************************************************************************
'MemoryPool::MemoryPool(void)
'{
'	m_rgchMemBlock = new char[MEMORYSIZE];
'	m_ichOffsetMemBlock = 0;
'	m_dwOwner = (DWORD)-1;
'}

 constructor MemoryPool()
  this.m_rgchMemBlock =  new CHAR[MEMORYSIZE]
 end constructor

'//
'// An empty destructor - see reasoning below
'//
'MemoryPool::~MemoryPool(void)
'{
'}
  destructor MemoryPool() 
    
  end destructor
'***********************************************************************************
'//
'// Unable to delete the memory block when we delete the pool,
'// as it may be still be in use due to a GrowPools() call; this
'// method will actually delete the pool's memory
'//
'***********************************************************************************
'void MemoryPool::ClearPool(void)
'{
'	delete [] m_rgchMemBlock;
'}
 sub MemoryPool.ClearPool()
   delete [] m_rgchMemBlock
  
 end sub
 '***********************************************************************************
'//
'// Advances the index forward by the given number of bytes.
'// Should there not be enough memory, or the number of bytes
'// is not allowed, this method will return 0. Can be called
'// and used exactly as malloc().
'//
'***********************************************************************************
'LPSTR MemoryPool::GetTempMemory(size_t cBytes)
'{
'	LPSTR lpMemory;

'	if (m_ichOffsetMemBlock + cBytes > MEMORYSIZE || cBytes <= 0)
'	{
'		return 0;
'	}
'	else
'	{
'		lpMemory = (LPSTR) m_rgchMemBlock + m_ichOffsetMemBlock;
'		m_ichOffsetMemBlock += cBytes;

'		return lpMemory;
'	}
'}
 'LPSTR MemoryPool::GetTempMemory(size_t cBytes)
 function MemoryPool.GetTempMemory(ctBytes as size_t) as LPSTR

	dim lpMemory as  LPSTR 

	if (m_ichOffsetMemBlock + ctBytes > MEMORYSIZE or  ctBytes <= 0)    then
	
		return 0
	
	else
	
		lpMemory = cast(LPSTR,m_rgchMemBlock) + m_ichOffsetMemBlock
		m_ichOffsetMemBlock += ctBytes

		return lpMemory
end if
end function
'***********************************************************************************
'//
'// Frees all the temporary memory by setting the index for
'// available memory back to the beginning
'//
'***********************************************************************************
'void MemoryPool::FreeAllTempMemory()
'{
'	m_ichOffsetMemBlock = 0;
'}

sub  MemoryPool.FreeAllTempMemory()
this.m_ichOffsetMemBlock = 0
end sub




         
        
         

