'//***************************************************************************
'// File:        MemoryManager.h
'//
'// Purpose:     Class definition for the memory manager used in the framework
'//              library.  Includes exported methods for accessing the class
'//              in C.
'// 
'// Platform:    Microsoft Windows
'//
'///***************************************************************************

'#ifdef __cplusplus

#include once "MemoryPool.bi"

'//
'// Total number of memory allocation pools to manage
'//

#define MEMORYPOOLS 4

'class MemoryManager
'{
'public:
'	MemoryManager(void);
'	~MemoryManager(void);

'	static MemoryManager* GetManager();

'	LPSTR CPP_GetTempMemory(size_t cByte);
'	void CPP_FreeAllTempMemory();

'private:
'	MemoryPool* CreateNewPool(DWORD dwThreadID);
'	MemoryPool* GetMemoryPool(DWORD dwThreadID);
'	void GrowPools();

'	int m_impCur;		// Current number of pools
'	int m_impMax;		// Max number of mem pools
'	MemoryPool* m_rgmp;	// Storage for the memory pools
'};

type MemoryManager   'extends object

public:
	declare constructor() 
	declare destructor() 

	declare static function GetManager() as  MemoryManager ptr
  declare function  CPP_GetTempMemory(ctByte as size_t) as lpstr
	declare sub   CPP_FreeAllTempMemory
	

private:
  declare function CreateNewPool(dwThreadID as DWORD) as  MemoryPool ptr
  declare function GetMemoryPool(dwThreadID  as DWORD ) as  MemoryPool ptr
	declare sub  GrowPools()
	
  dim m_impCur as long ';		// Current number of pools
	dim  m_impMax as long ';		// Max number of mem pools
	dim  m_rgmp as MemoryPool ptr ';	// Storage for the memory pools
end type

'//
'// Defines functions for accessing class from a C projects
'//

extern "C"

	declare function MGetTempMemory(ctByte as size_t ) as LPSTR
	declare sub  MFreeAllTempMemory()
end extern


'///***************************************************************************
'// File:        MemoryManager.cpp
'//
'// Purpose:     The memory manager class is an update to the memory manager
'//              in the previous release of the framework.  This class provides
'//              each thread with an array of bytes to use as temporary memory.
'//              The size of the array, and the methods for dealing with the
'//              memory explicitly, is in the class MemoryPool.  
'//
'//              MemoryManager handles assigning of threads to pools, and the
'//              creation of new pools when a thread asks for memory the first
'//              time.  Using a singleton class, the manager provides an interface
'//              to C code into the manager.  The number of unique pools starts
'//              as MEMORYPOOLS, defined in MemoryManager.h.  When a new thread
'//              needs a pool, and the current set of pools are all assigned,
'//              the number of pools increases by a factor of two.
'// 
'// Platform:    Microsoft Windows
'//
'///***************************************************************************

'#include "MemoryManager.h"

'//
'// Singleton instance of the class
'//
'MemoryManager* vpmm;
 dim shared vpmm as MemoryManager ptr
'***********************************************************************************
'// Interface for C callers to ask for memory
'***********************************************************************************
'// See MemoryPool.h for more details
'//
'LPSTR MGetTempMemory(size_t cByte)
'{
'	return MemoryManager::GetManager()->CPP_GetTempMemory(cByte);
'}
function MGetTempMemory(ctByte as size_t) as LPSTR
  return MemoryManager.GetManager()->CPP_GetTempMemory(ctByte)
end function
'***********************************************************************************
'// Interface for C callers to allow their memory to be reused
'***********************************************************************************
'// See MemoryPool.h for more details
'//
'void MFreeAllTempMemory()
'{
'	MemoryManager::GetManager()->CPP_FreeAllTempMemory();
'}
sub  MFreeAllTempMemory()

	MemoryManager.GetManager()->CPP_FreeAllTempMemory()
end sub
'***********************************************************************************
'// Returns the singleton class, or creates one if it doesn't exit
'***********************************************************************************
'MemoryManager* MemoryManager::GetManager()
'{
'	if (!vpmm)
'	{
'		vpmm = new MemoryManager();
'	}
'	return vpmm;
'}
function MemoryManager.GetManager()   as MemoryManager ptr
           
if vpmm =  false then   'not initiated
  vpmm = new MemoryManager()
end if
	return vpmm
end function
'***********************************************************************************
'// Default constructor
'***********************************************************************************
'MemoryManager::MemoryManager(void)
'{
'	m_impCur = 0;
'	m_impMax = MEMORYPOOLS;
'	m_rgmp = new MemoryPool[MEMORYPOOLS];
'}
 constructor MemoryManager()

	this.m_impCur = 0
  this.m_impMax = MEMORYPOOLS
  this.m_rgmp = new MemoryPool[MEMORYPOOLS]
end constructor
'***********************************************************************************
'// Destructor.  Because of the way memory pools get copied,
'// this function needs to call an additional function to clear
'// up the MemoryPool memory - the deconstructor on MemoryPool
'// does not actually delete its memory
'***********************************************************************************
'MemoryManager::~MemoryManager(void)
'{
'	MemoryPool* pmp = m_rgmp;
'	int i;

'	for (i = 0; i < m_impMax; i++)
'	{
'		if (pmp->m_rgchMemBlock)
'		{
'			pmp->ClearPool();
'		}
'		pmp++;
'	}
'	delete [] m_rgmp;
'}
 destructor MemoryManager()
   dim pmp as MemoryPool ptr  = this.m_rgmp
   

  
   dim i as long
   for i =  0 to m_impMax - 1
     if pmp-> m_rgchMemBlock then
       pmp->ClearPool()
     end if
    pmp += 1
   next
   delete [] m_rgmp
 end destructor

'***********************************************************************************
'// Method that will query the correct memory pool of the calling
'// thread for a set number of bytes.  Returns 0 if there was a
'// failure in getting the memory.
'***********************************************************************************
'LPSTR MemoryManager::CPP_GetTempMemory(size_t cByte)
'{
'	DWORD dwThreadID;
'	MemoryPool* pmp;

'	dwThreadID = GetCurrentThreadId(); //the id of the calling thread
'	pmp = GetMemoryPool(dwThreadID);

'	if (!pmp) //no more room for pools
'	{
'		return 0;
'	}

'	return pmp->GetTempMemory(cByte);
'}
 function MemoryManager.CPP_GetTempMemory(ctByte as size_t ) as LPSTR
  DIM  dwThreadID as DWORD
  Dim pmp as  MemoryPool PTR 
  dwThreadID = GetCurrentThreadId() '//the id of the calling thread 
  pmp = GetMemoryPool(dwThreadID)
   
  if  pmp = false  then  '//no more room for pools
    return 0
    
  end if
   
  return  pmp->GetTempMemory(ctByte)
 end function

'***********************************************************************************
'// Method that tells the pool owned by the calling thread that
'// it is free to reuse all of its memory
'***********************************************************************************
'void MemoryManager::CPP_FreeAllTempMemory()
'{
'	DWORD dwThreadID;
'	MemoryPool* pmp;

'	dwThreadID = GetCurrentThreadId(); //the id of the calling thread
'	pmp = GetMemoryPool(dwThreadID);

'	if (!pmp) //no more room for pools
'	{
'		return;
'	}

'	pmp->FreeAllTempMemory();
'}
 sub MemoryManager.CPP_FreeAllTempMemory()
   DIM  dwThreadID as DWORD
  Dim pmp as  MemoryPool PTR 
  if pmp = false   then  '//no more room for pools
    return 
    
  end if 
   pmp->FreeAllTempMemory
 end sub

'***********************************************************************************
'// Method iterates through the memory pools in an attempt to find
'// the pool that matches the given thread ID. If a pool is not found,
'// it creates a new one
'***********************************************************************************
'MemoryPool* MemoryManager::GetMemoryPool(DWORD dwThreadID)
'{
'	int imp; //loop var
'	MemoryPool* pmp; //current pool

'	pmp = m_rgmp;

'	for (imp = 0; imp < m_impCur; imp++)
'	{
'		if (pmp->m_dwOwner == dwThreadID)
'		{
'			return pmp;
'		}

'		pmp++;
'	}

'	return CreateNewPool(dwThreadID); //didn't find the owner, make a new one
'}
 function  MemoryManager.GetMemoryPool(dwThreadID as DWORD )  as MemoryPool PTR
'{
dim limp as long ' //loop var
dim pmp as 	MemoryPool ptr ' //current pool

	pmp = m_rgmp

	for limp = 0 to  m_impCur-1 

		if  pmp->m_dwOwner =  dwThreadID then return pmp

next

return CreateNewPool(dwThreadID) ' //didn't find the owner, make a new one

end function
'***********************************************************************************
'// Will assign an unused pool to a thread; should all pools be assigned,
'// it will grow the number of pools available.
'***********************************************************************************
'MemoryPool* MemoryManager::CreateNewPool(DWORD dwThreadID)
'{
'	if (m_impCur >= m_impMax)
'	{
'		GrowPools();
'	}
'	m_rgmp[m_impCur++].m_dwOwner = dwThreadID;

'	return m_rgmp+m_impCur-1;
'}

 function  MemoryManager.CreateNewPool( dwThreadID as DWORD )   as  MemoryPool ptr

	if m_impCur >= m_impMax then GrowPools()
  m_impCur += 1
	m_rgmp[m_impCur].m_dwOwner = dwThreadID

	return (m_rgmp + m_impCur-1)
end function

'***********************************************************************************
'// Increases the number of available pools by a factor of two. All of
'// the old pools have their memory pointed to by the new pools. The
'// memory for the new pools that get replaced is first freed. The reason
'// ~MemoryPool() can't free its array is in this method - they would be
'// deleted when the old array of pools is freed at the end of the method,
'// despite the fact they are now being pointed to by the new pools.
'***********************************************************************************
'void MemoryManager::GrowPools()
'{
'	MemoryPool* rgmpTemp;
'	MemoryPool* pmpDst;
'	MemoryPool* pmpSrc;

'	int i, impMaxNew;

'	impMaxNew = 2*m_impMax;
'	pmpDst = rgmpTemp = new MemoryPool[2*m_impMax];
'	pmpSrc = m_rgmp;

'	for (i = 0; i < m_impCur; i++)
'	{
'		delete [] pmpDst->m_rgchMemBlock;
'		pmpDst->m_rgchMemBlock = pmpSrc->m_rgchMemBlock;
'		pmpDst->m_dwOwner = pmpSrc->m_dwOwner;

'		pmpDst++;
'		pmpSrc++;
'	}
'	delete [] m_rgmp;
'	m_rgmp = rgmpTemp;
'	m_impMax = impMaxNew;
'}

sub  MemoryManager.GrowPools()

dim as Memorypool PTR rgmpTemp, pmpDst,pmpSrc
dim as long i,impMaxNew 
impMaxNew = 2*m_impMax
rgmpTemp = new MemoryPool[2*m_impMax]
pmpDst = rgmpTemp
pmpSrc = m_rgmp

	for i = 0 to   m_impCur - 1

	delete [] pmpDst->m_rgchMemBlock
pmpDst->m_rgchMemBlock = pmpSrc->m_rgchMemBlock
pmpDst->m_dwOwner = pmpSrc->m_dwOwner

	pmpDst += 1
	pmpSrc += 1
	next
  
	delete [] m_rgmp
	m_rgmp = rgmpTemp
	m_impMax = impMaxNew
end sub


' ///***************************************************************************
'// GetTempMemory()
'//
'// Purpose: 
'//       Allocates temporary memory. Temporary memory
'//       can only be freed in one chunk, by calling
'//       FreeAllTempMemory(). This is done by Excel12f().
'//
'// Parameters:
'//
'//      size_t cBytes      How many bytes to allocate
'//
'// Returns: 
'//
'//      LPSTR           A pointer to the allocated memory,
'//                      or 0 if more memory cannot be
'//                      allocated. If this fails,
'//                      check that MEMORYSIZE is big enough
'//                      in MemoryPool.h and that you have
'//                      remembered to call FreeAllTempMemory
'//
'// Comments:
'//
'//	Algorithm:
'//       
'//      The memory allocation algorithm is extremely
'//      simple: on each call, allocate the next cBytes
'//      bytes of a static memory buffer. If the buffer
'//      becomes too full, simply fail. To free memory,
'//      simply reset the pointer (vOffsetMemBlock)
'//      back to zero. This memory scheme is very fast
'//      and is optimized for the assumption that the
'//      only thing you are using temporary memory
'//      for is to hold arguments while you call Excel12f().
'//      We rely on the fact that you will free all the
'//      temporary memory at the same time. We also
'//      assume you will not need more memory than
'//      the amount required to hold a few arguments
'//      to Excel12f().
'// 
'//      Note that the memory allocation algorithm
'//      now supports multithreaded applications by
'//      giving each unique thread its own static
'//      block of memory. This is handled in the
'//      MemoryManager.cpp file automatically.
'//
'///***************************************************************************

function  GetTempMemory(ctBytes as size_t )  as LPSTR

	return MGetTempMemory(ctBytes)
end function

'///***************************************************************************
'// FreeAllTempMemory()
'//
'// Purpose: 
'//
'//      Frees all temporary memory that has been allocated
'//      for the current thread
'//
'// Parameters:
'//
'// Returns: 
'//
'// Comments:
'//
'//
'///***************************************************************************

sub FreeAllTempMemory()

	MFreeAllTempMemory()
end sub

